#include <Ice/Ice.h>
#include <MWSessionI.h>
#include <list>
#include <MWMap.h>

using namespace std;
using namespace MW;

MWMap*
MWMap::instance()
{
    static MWMap mwMapInstance;
    return &mwMapInstance;
}

void
MWMap::enter(const shared_ptr<MWSessionPrx>& session, const shared_ptr<MWCallbackPrx>& callback,
	     const Ice::Current& current)
{
  lock_guard<mutex> sync(_mutex);
  _callbacks.push_back(callback);

  auto p = _connectionMap.find(current.con);
  if (p == _connectionMap.end())
    {
      cout << "Enter: create a new entry in connection map" << endl;
      _connectionMap[current.con].push_back(session);

      //
      // Never close this connection from Glacier2 and turn on heartbeats with a timeout of 30s
      //
      current.con->setACM(30, Ice::ACMClose::CloseOff, Ice::ACMHeartbeat::HeartbeatAlways);
      current.con->setCloseCallback([this](const shared_ptr<Ice::Connection>& con)
				    {
				      
				      deadRouter(con);
				      
				    });
    }
  else
    {
      cout << "enter: add session to existing connection map entry" << endl;
      p->second.push_back(session);

    }
}

void
MWMap::leave(const shared_ptr<MWCallbackPrx>& callback, const Ice::Current& current)
{
  lock_guard<mutex> sync(_mutex);
  _callbacks.remove_if([&callback](const shared_ptr<MWCallbackPrx>& cb) { return Ice::proxyIdentityEqual(callback, cb); });
  _connectionMap[current.con].remove_if([&current](const shared_ptr<MWSessionPrx>& s) {return current.id == s->ice_getIdentity(); });
}

void
MWMap::message(const string& data) const
{
  lock_guard<mutex> sync(const_cast<mutex&>(_mutex));
  for(const auto& cb: _callbacks)
    {
      cb->messageAsync(data);
    }
}

void
MWMap::deadRouter(const shared_ptr<Ice::Connection>& con)
{
  cout << "Detected dead router. Detroying all sessions " << endl;
  list<shared_ptr<MWSessionPrx>> sessions;
  lock_guard<mutex> sync(_mutex);
  auto p = _connectionMap.find(con);
  if(p != _connectionMap.end())
    {
      sessions.swap(p->second);
      _connectionMap.erase(p);
    }
  for(auto& s: sessions )
    {
      s->destroy();
    }
}

void
MWMap::destroy()
{
  lock_guard<mutex> sync(_mutex);
  _callbacks.clear();
  _connectionMap.clear();
    
}
