#include <Ice/Ice.h>
#include <Glacier2/Glacier2.h>
#include <MW.h>
#include <MWSessionI.h>

using namespace std;
using namespace MW;

class DummyPermissionsVerifierI : public Glacier2::PermissionsVerifier
{
public:
  
  virtual bool    
  checkPermissions(string userId, string password, string&, const Ice::Current&) const override
  {
    cout << "User " << userId << " with password " << password << " has joined the map" << endl;
    return true;
  }
  
};

int
main(int argc, char* argv[])
{
  
#ifdef ICE_STATIC_LIBS
  Ice::registerIceWSS();
#endif

  try
    {
      Ice::CtrlCHandler ctrlCHandler;
      Ice::CommunicatorHolder ich(argc, argv, "config.server");
      
      auto communicator = ich.communicator();
      
      ctrlCHandler.setCallback(
			       [communicator](int)
			       {
				 communicator->shutdown();
			       });

      if(argc > 1)
        {
	  cerr << argv[0] << ": too many arguments" << endl;
        }
      else
        {
	  auto adapter = ich->createObjectAdapter("MWServer");
	  
	  auto dpv = make_shared<DummyPermissionsVerifierI>();
	  adapter->add(dpv, Ice::stringToIdentity("MWSessionVerifier"));

	  auto mwsm = make_shared<MWSessionManagerI>();
	  adapter->add(mwsm, Ice::stringToIdentity("MWSessionManager"));
	  
	  adapter->activate();
	  cout << "\n +++> Magic World server version 0.2 <+++ \n" << endl;

	  ich->waitForShutdown();
	  mwsm->destroy();
	  
	}      
    }
  catch(const std::exception& e)
    {
      cerr << e.what() << endl;
      return 1;
    }
  
  return 0;
}
